@@ -1,7 +1,7 @@
 """Define new Ops from existing Ops"""
 from collections import OrderedDict
 from functools import partial
-from typing import List, Optional
+from typing import List, Optional, Sequence, cast
 
 import aesara.tensor as at
 from aesara.compile.function.pfunc import rebuild_collect_shared
@@ -22,6 +22,7 @@ from aesara.graph.fg import FunctionGraph
 from aesara.graph.null_type import NullType
 from aesara.graph.op import HasInnerGraph, Op
 from aesara.graph.opt import in2out, local_optimizer
+from aesara.graph.utils import MissingInputError
 from aesara.tensor.basic_opt import ShapeFeature
 
 
@@ -39,8 +40,8 @@ def infer_shape(outs, inputs, input_shapes):
     # let it initialize itself with an empty fgraph, otherwise we will
     # need to do it manually
     for inp, inp_shp in zip(inputs, input_shapes):
-        if inp_shp is not None and len(inp_shp) != inp.ndim:
-            assert len(inp_shp) == inp.ndim
+        if inp_shp is not None and len(inp_shp) != inp.type.ndim:
+            assert len(inp_shp) == inp.type.ndim
 
     shape_feature = ShapeFeature()
     shape_feature.on_attach(FunctionGraph([], []))
@@ -334,8 +335,15 @@ class OpFromGraph(Op, HasInnerGraph):
                 raise TypeError(
                     f"Inputs and outputs must be Variable instances; got {i}"
                 )
-            if i in inputs and isinstance(i, Constant):
-                raise TypeError(f"Constants not allowed as inputs; {i}")
+            if i in inputs:
+                if isinstance(i, Constant):
+                    raise TypeError(f"Constants not allowed as inputs; {i}")
+                if isinstance(i, SharedVariable):
+                    raise TypeError(f"SharedVariables not allowed as inputs; {i}")
+
+        for var in graph_inputs(outputs, inputs):
+            if var not in inputs and not isinstance(var, (Constant, SharedVariable)):
+                raise MissingInputError(f"OpFromGraph is missing an input: {var}")
 
         if "updates" in kwargs or "givens" in kwargs:
             raise NotImplementedError("Updates and givens are not allowed here")
@@ -349,7 +357,7 @@ class OpFromGraph(Op, HasInnerGraph):
         shared_vars = [var.type() for var in self.shared_inputs]
 
         new = rebuild_collect_shared(
-            outputs,
+            cast(Sequence[Variable], outputs),
             inputs=inputs + shared_vars,
             replace=dict(zip(self.shared_inputs, shared_vars)),
             copy_inputs_over=False,
@@ -357,8 +365,9 @@ class OpFromGraph(Op, HasInnerGraph):
         (
             local_inputs,
             local_outputs,
-            [clone_d, update_d, update_expr, shared_inputs],
+            (clone_d, update_d, update_expr, shared_inputs),
         ) = new
+
         assert len(local_inputs) == len(inputs) + len(self.shared_inputs)
         assert len(local_outputs) == len(outputs)
         assert not update_d
@@ -372,6 +381,11 @@ class OpFromGraph(Op, HasInnerGraph):
         self.kwargs = kwargs
         self.input_types = [inp.type for inp in inputs]
         self.output_types = [out.type for out in outputs]
+
+        self.lop_overrides = lop_overrides
+        self.grad_overrides = grad_overrides
+        self.rop_overrides = rop_overrides
+
         if lop_overrides != "default":
             if grad_overrides != "default":
                 raise ValueError(
@@ -729,19 +743,87 @@ class OpFromGraph(Op, HasInnerGraph):
         ]
         return ret_l
 
-    def make_node(self, *inputs):
+    def __call__(self, *inputs, **kwargs):
+        # The user interface doesn't expect the shared variable inputs of the
+        # inner-graph, but, since `Op.make_node` does (and `Op.__call__`
+        # dispatches to `Op.make_node`), we need to compensate here
         num_expected_inps = len(self.inner_inputs) - len(self.shared_inputs)
-        if len(inputs) != num_expected_inps:
-            raise ValueError(
-                f"Expected {int(num_expected_inps)} inputs, got {len(inputs)}"
-            )
-        inputs = [
-            inp_t.filter_variable(inp) for inp, inp_t in zip(inputs, self.input_types)
+
+        if len(inputs) == num_expected_inps:
+            actual_inputs = inputs + tuple(self.shared_inputs)
+            return super().__call__(*actual_inputs, **kwargs)
+        elif len(inputs) == len(self.inner_inputs):
+            return super().__call__(*inputs, **kwargs)
+        else:
+            raise ValueError(f"Expected at least {num_expected_inps} input(s)")
+
+    def make_node(self, *inputs):
+        # The `inputs` received here should correspond to the inputs in the
+        # `Apply` nodes we produce below
+        if len(inputs) != len(self.inner_inputs):
+            raise ValueError(f"Expected {len(self.inner_inputs)} input(s)")
+
+        num_expected_inps = len(self.inner_inputs) - len(self.shared_inputs)
+        non_shared_inputs = inputs[:num_expected_inps]
+
+        non_shared_inputs = [
+            inp_t.filter_variable(inp)
+            for inp, inp_t in zip(non_shared_inputs, self.input_types)
         ]
+
+        new_shared_inputs = inputs[num_expected_inps:]
+        inner_and_input_shareds = list(zip(self.shared_inputs, new_shared_inputs))
+
+        if not all(inp_s == inn_s for inn_s, inp_s in inner_and_input_shareds):
+            # The shared variables are not equal to the original shared
+            # variables, so we construct a new `Op` that uses the new shared
+            # variables instead.
+            # All this is really doing is making the unused (internally, at
+            # least) `self.outputs` and `self.shared_inputs` consistent.
+            # We could just as easily `copy` this `Op`, update
+            # `self.shared_inputs`, and avoid cloning anything, but this is a
+            # more "change-proof" approach, because it still work when/if those
+            # attributes end up being used.
+            replace = dict(inner_and_input_shareds)
+
+            # If the new shared variables are inconsistent with the inner-graph,
+            # such errors should arise in this step
+            new_inner_outputs = clone_replace(
+                self.outputs, replace=replace, share_inputs=True
+            )
+
+            # `self.inputs` should not contain any shared variables, so we know
+            # that those are inputs to `new_outputs`, because we chose not to
+            # clone inputs; however, it's possible that the new shared variable
+            # inputs aren't actually shared variables.  When they aren't we
+            # need to add them as new inputs.
+            unshared_inputs = [
+                inp for inp in new_shared_inputs if not isinstance(inp, SharedVariable)
+            ]
+            new_inner_inputs = self.inputs + unshared_inputs
+
+            new_op = type(self)(
+                inputs=new_inner_inputs,
+                outputs=new_inner_outputs,
+                inline=self.is_inline,
+                lop_overrides=self.lop_overrides,
+                grad_overrides=self.grad_overrides,
+                rop_overrides=self.rop_overrides,
+                connection_pattern=self._connection_pattern,
+                name=self.name,
+                **self.kwargs,
+            )
+            new_inputs = (
+                list(non_shared_inputs) + unshared_inputs + new_op.shared_inputs
+            )
+        else:
+            new_op = self
+            new_inputs = list(non_shared_inputs) + new_op.shared_inputs
+
         apply_node = Apply(
-            self,
-            list(inputs) + self.shared_inputs,
-            [type() for type in self.output_types],
+            new_op,
+            new_inputs,
+            [type() for type in new_op.output_types],
         )
         return apply_node
 
@@ -854,7 +936,7 @@ def inline_ofg_expansion(fgraph, node):
 optdb.register(
     "inline_ofg_expansion",
     in2out(inline_ofg_expansion),
-    -0.01,
     "fast_compile",
     "fast_run",
+    position=-0.01,
 )
