@@ -9,7 +9,7 @@ import logging
 import time
 import warnings
 from itertools import chain
-from typing import List
+from typing import List, Tuple, Type
 
 import numpy as np
 
@@ -26,18 +26,16 @@ from aesara.graph.basic import (
     graph_inputs,
 )
 from aesara.graph.destroyhandler import DestroyHandler
-from aesara.graph.features import PreserveVariableAttributes
-from aesara.graph.fg import FunctionGraph, InconsistencyError
+from aesara.graph.features import Feature, PreserveVariableAttributes
+from aesara.graph.fg import FunctionGraph
 from aesara.graph.op import HasInnerGraph
-from aesara.graph.utils import get_variable_trace_string
+from aesara.graph.utils import InconsistencyError, get_variable_trace_string
 from aesara.link.basic import Container
 from aesara.link.utils import raise_with_op
 
 
 _logger = logging.getLogger("aesara.compile.function.types")
 
-__docformat__ = "restructuredtext en"
-
 
 class UnusedInputError(Exception):
     """
@@ -150,18 +148,23 @@ class Supervisor:
                 raise InconsistencyError(f"Trying to destroy a protected variable: {r}")
 
 
-def std_fgraph(input_specs, output_specs, accept_inplace=False):
-    """
-    Makes an FunctionGraph corresponding to the input specs and the output
-    specs.  Any SymbolicInput in the input_specs, if its update field
-    is not None, will add an output to the FunctionGraph corresponding to that
-    update. The return value is the FunctionGraph as well as a list of
-    SymbolicOutput instances corresponding to the updates.
+def std_fgraph(
+    input_specs: List[SymbolicInput],
+    output_specs: List[SymbolicOutput],
+    accept_inplace: bool = False,
+    features: List[Type[Feature]] = [PreserveVariableAttributes],
+) -> Tuple[FunctionGraph, List[SymbolicOutput]]:
+    """Make or set up `FunctionGraph` corresponding to the input specs and the output specs.
 
-    If accept_inplace is False, the graph will be checked for inplace
+    Any `SymbolicInput` in the `input_specs`, if its `update` field is not
+    ``None``, will add an output corresponding to that update to the
+    `FunctionGraph`. The return value is the `FunctionGraph` as well as a list
+    of `SymbolicOutput` instances corresponding to the updates.
+
+    If `accept_inplace` is ``False``, the graph will be checked for in-place
     operations and an exception will be raised if it has any. If
-    accept_inplace is True, a DestroyHandler will be added to the FunctionGraph
-    if there are any inplace operations.
+    `accept_inplace` is ``True``, a `DestroyHandler` will be added to the
+    `FunctionGraph` if there are any in-place operations.
 
     The returned FunctionGraph is a clone of the graph between the provided
     inputs and outputs.
@@ -169,8 +172,8 @@ def std_fgraph(input_specs, output_specs, accept_inplace=False):
     """
     orig_inputs = [spec.variable for spec in input_specs]
 
-    # Extract the updates and the mapping between update outputs and
-    # the updated inputs.
+    # Extract the updates and the mapping between update outputs and the
+    # updated inputs
     updates = []
     update_mapping = {}
     out_idx = len(output_specs)
@@ -205,14 +208,11 @@ def std_fgraph(input_specs, output_specs, accept_inplace=False):
     )
 
     # If named nodes are replaced, keep the name
-    for feature in std_fgraph.features:
+    for feature in features:
         fgraph.attach_feature(feature())
     return fgraph, list(map(SymbolicOutput, updates))
 
 
-std_fgraph.features = [PreserveVariableAttributes]
-
-
 class AliasedMemoryError(Exception):
     """
     Memory is aliased that should not be.
@@ -220,12 +220,8 @@ class AliasedMemoryError(Exception):
     """
 
 
-###
-# Function
-###
-
-# unique id object used as a placeholder for duplicate entries
-DUPLICATE = ["DUPLICATE"]
+# A sentinel for duplicate entries
+DUPLICATE = object()
 
 
 class Function:
@@ -530,15 +526,14 @@ class Function:
         self._value = ValueAttribute()
         self._container = ContainerAttribute()
 
-        # Compute self.n_returned_outputs.
-        # This is used only when fn.need_update_inputs is False
-        # because we're using one of the VM objects and it is
-        # putting updates back into the input containers all by itself.
         assert len(self.maker.expanded_inputs) == len(self.input_storage)
-        self.n_returned_outputs = len(self.output_storage)
-        for input in self.maker.expanded_inputs:
-            if input.update is not None:
-                self.n_returned_outputs -= 1
+
+        # This is used only when `fn.need_update_inputs` is `False`, because
+        # we're using one of the VM objects and it is putting updates back into
+        # the input containers all by itself.
+        self.n_returned_outputs = len(self.output_storage) - sum(
+            inp.update is not None for inp in self.maker.expanded_inputs
+        )
 
         for node in self.maker.fgraph.apply_nodes:
             if isinstance(node.op, HasInnerGraph):
@@ -653,7 +648,7 @@ class Function:
         )
 
         # Re initialize Outs and swap update and variable in Ins
-        # By doing this, we can pass FunctionMaker._check_unused_inputs()
+        # By doing this, we can pass FunctionMaker.check_unused_inputs()
         outs = list(map(SymbolicOutput, fg_cpy.outputs[: len(maker.outputs)]))
         for out_ori, out_cpy in zip(maker.outputs, outs):
             out_cpy.borrow = out_ori.borrow
@@ -843,8 +838,8 @@ class Function:
             # Set positional arguments
             i = 0
             for arg in args:
-                # TODO: provide a Param option for skipping the filter if we
-                #      really want speed.
+                # TODO: provide a option for skipping the filter if we really
+                # want speed.
                 s = self.input_storage[i]
                 # see this emails for a discuation about None as input
                 # https://groups.google.com/group/theano-dev/browse_thread/thread/920a5e904e8a8525/4f1b311a28fc27e5
@@ -1102,13 +1097,8 @@ class Function:
         return [i.variable for i in self.maker.inputs if i.implicit]
 
     def sync_shared(self):
-        if hasattr(aesara, "gpuarray") and aesara.gpuarray.pygpu_activated:
-            import pygpu
-
-            for i in self.maker.fgraph.update_mapping.values():
-                inp = self.input_storage[i]
-                if isinstance(inp.data, pygpu.gpuarray.GpuArray):
-                    inp.data.sync()
+        # NOTE: sync was needed on old gpu backend
+        pass
 
 
 # pickling/deepcopy support for Function
@@ -1139,7 +1129,7 @@ def _pickle_Function(f):
                 ):
                     if np.may_share_memory(d_i, d_j):
                         if f.pickle_aliased_memory_strategy == "warn":
-                            _logger.warning(
+                            warnings.warn(
                                 "aliased relationship between "
                                 f"Function arguments {d_i}, {d_j} "
                                 "will not be preserved by "
@@ -1173,25 +1163,21 @@ def _constructor_Function(maker, input_storage, inputs_data, trust_input=False):
 copyreg.pickle(Function, _pickle_Function)
 
 
-###
-# FunctionMaker
-###
 def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
-    """
-    Insert deepcopy in the fgraph to break aliasing of outputs
-    """
-    # This loop was inserted to remove aliasing between outputs when
-    # they all evaluate to the same value. Originally it was OK for
-    # outputs to be aliased, but some of the outputs can be shared
-    # variables, and is not good for shared variables to be
-    # aliased. It might be possible to optimize this by making sure
-    # there is no aliasing only between shared variables.
+    """Insert deepcopy in the fgraph to break aliasing of outputs.
 
-    # If some outputs are constant, we add deep copy to respect the
-    # memory contract
+    This loop was inserted to remove aliasing between outputs when they all
+    evaluate to the same value. Originally it was OK for outputs to be aliased,
+    but some of the outputs can be shared variables, and is not good for shared
+    variables to be aliased. It might be possible to optimize this by making
+    sure there is no aliasing only between shared variables.
 
-    # We don't insert deep copy when the output.borrow is True for all
-    # concerned outputs.
+    If some outputs are constant, we add deep copy to respect the memory
+    contract
+
+    We don't insert deep copy when :attr:`SymbolicOutput.borrow` is ``True``
+    for all concerned outputs.
+    """
 
     assert len(wrapped_inputs) == len(fgraph.inputs)
     assert len(wrapped_outputs) == len(fgraph.outputs)
@@ -1247,7 +1233,7 @@ def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
                                 "output",
                                 i,
                                 view_op(fgraph.outputs[i]),
-                                reason="insert_deepcopy",
+                                reason=reason,
                             )
                             break
                         else:
@@ -1255,7 +1241,7 @@ def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
                                 "output",
                                 i,
                                 deep_copy_op(fgraph.outputs[i]),
-                                reason="insert_deepcopy",
+                                reason=reason,
                             )
                             break
                     elif wrapped_outputs[i].borrow:
@@ -1263,7 +1249,7 @@ def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
                             "output",
                             i,
                             view_op(fgraph.outputs[i]),
-                            reason="insert_deepcopy",
+                            reason=reason,
                         )
                         break
                     else:
@@ -1271,14 +1257,11 @@ def insert_deepcopy(fgraph, wrapped_inputs, wrapped_outputs):
                             "output",
                             i,
                             deep_copy_op(fgraph.outputs[i]),
-                            reason="insert_deepcopy",
+                            reason=reason,
                         )
                         break
 
 
-NODEFAULT = ["NODEFAULT"]
-
-
 class FunctionMaker:
     """
     `FunctionMaker` is the class to `create` `Function` instances.
@@ -1335,20 +1318,6 @@ class FunctionMaker:
                 "instance"
             )
 
-    @staticmethod
-    def expand_in(sinput, rinputs):
-        # For SymbolicInputKits, this extracts a list of SymbolicInput
-        # instances and corresponding indices such that these
-        # SymbolicInputs are representative of some of the Variable
-        # instances in inputs.  For SymbolicInput, this returns None
-        # as the list of indices and a list with just the
-        # SymbolicInput.
-        # if isinstance(sinput, SymbolicInputKit):
-        #    return sinput.complete(rinputs)
-        # elif isinstance(sinput, SymbolicInput):
-        if isinstance(sinput, SymbolicInput):
-            return [None, [sinput]]
-
     @staticmethod
     def wrap_out(output):
         if isinstance(output, SymbolicOutput):
@@ -1358,186 +1327,8 @@ class FunctionMaker:
         else:
             raise TypeError(f"Unknown output type: {type(output)} ({output})")
 
-    def __init__(
-        self,
-        inputs,
-        outputs,
-        mode=None,
-        accept_inplace=False,
-        function_builder=Function,
-        profile=None,
-        on_unused_input=None,
-        fgraph=None,
-        output_keys=None,
-        name=None,
-    ):
-        # Save the provided mode, not the instantiated mode.
-        # The instantiated mode don't pickle and if we unpickle an Aesara
-        # function and it get re-compiled, we want the current optimizer to be
-        # used, not the optimizer when it was saved.
-        self.mode = mode
-        mode = aesara.compile.mode.get_mode(mode)
-
-        # Assert old way of working isn't used
-        if getattr(mode, "profile", None):
-            raise TypeError("profile passed via 'mode'. This isn't supported anymore")
-        self.profile = profile
-        if profile:
-            # This is very important:
-            # 1) We preload the cache here to not have its timing
-            #    included in optimization that compile function.
-            # 2) Do not refresh the cache here by default. It cause
-            #    too much execution time during testing as we compile
-            #    much more functions then the number of compile c
-            #    module.
-            aesara.link.c.basic.get_module_cache().refresh()
-        # Handle the case where inputs and/or outputs is a single
-        # Variable (not in a list)
-        unpack_single = False
-        return_none = False
-        if outputs is None:
-            return_none = True
-            outputs = []
-        if not isinstance(outputs, (list, tuple)):
-            unpack_single = True
-            outputs = [outputs]
-        if not isinstance(inputs, (list, tuple)):
-            inputs = [inputs]
-
-        # Wrap them in In or Out instances if needed.
-        inputs = [self.wrap_in(i) for i in inputs]
-        outputs = [self.wrap_out(o) for o in outputs]
-        _inputs = list(
-            graph_inputs(
-                [o.variable for o in outputs]
-                + [i.update for i in inputs if getattr(i, "update", False)]
-            )
-        )
-
-        # Check if some input variables are unused
-        self._check_unused_inputs(inputs, outputs, on_unused_input)
-
-        # Make a list of (SymbolicInput|SymblicInputKits, indices,
-        # [SymbolicInput,...]), one tuple for each input. (See
-        # Function.indices for more details)
-        indices = [[input] + self.expand_in(input, _inputs) for input in inputs]
-
-        if fgraph is None:
-            need_opt = True
-            # make the fgraph (copies the graph, creates NEW INPUT AND
-            # OUTPUT VARIABLES)
-            fgraph, additional_outputs = std_fgraph(inputs, outputs, accept_inplace)
-            fgraph.profile = profile
-        else:
-            # fgraph is already an optimized one
-            need_opt = False
-            updates = [spec.update for spec in inputs if spec.update]
-            additional_outputs = list(map(SymbolicOutput, updates))
-
-        self.fgraph = fgraph
-
-        # Fetch the optimizer and linker
-        optimizer, linker = mode.optimizer, copy.copy(mode.linker)
-        if need_opt:
-            # Why we add stack on node when it get done in output var?
-            try:
-                start_optimizer = time.time()
-
-                # In case there is an error during optimization.
-                optimizer_profile = None
-                opt_time = None
-
-                with config.change_flags(
-                    compute_test_value=config.compute_test_value_opt,
-                    traceback__limit=config.traceback__compile_limit,
-                ):
-                    optimizer_profile = optimizer(fgraph)
-
-                    end_optimizer = time.time()
-                    opt_time = end_optimizer - start_optimizer
-                    _logger.debug(f"Optimizing took {opt_time:f} seconds")
-
-                    # Add deep copy to respect the memory interface
-                    insert_deepcopy(fgraph, inputs, outputs + additional_outputs)
-            finally:
-
-                # If the optimizer got interrupted
-                if opt_time is None:
-                    end_optimizer = time.time()
-                    opt_time = end_optimizer - start_optimizer
-
-                aesara.compile.profiling.total_graph_opt_time += opt_time
-
-                if profile:
-                    if optimizer_profile is None and hasattr(optimizer, "pre_profile"):
-                        optimizer_profile = optimizer.pre_profile
-
-                    profile.optimizer_time += opt_time
-
-                    if config.profile_optimizer:
-                        profile.optimizer_profile = (optimizer, optimizer_profile)
-                # IF False, if mean the profile for that function was
-                # explicitly disabled
-                elif config.profile_optimizer and profile is not False:
-                    warnings.warn(
-                        (
-                            "config.profile_optimizer requires config.profile to "
-                            " be set to True as well"
-                        ),
-                        stacklevel=3,
-                    )
-
-        # initialize the linker
-        if not hasattr(linker, "accept"):
-            raise ValueError(
-                "'linker' parameter of FunctionMaker should be "
-                f"a Linker with an accept method or one of {list(aesara.compile.mode.predefined_linkers.keys())}"
-            )
-
-        # the 'no_borrow' outputs are the ones for which that we can't
-        # return the internal storage pointer.
-        assert len(fgraph.outputs) == len(outputs + additional_outputs)
-        no_borrow = [
-            output
-            for output, spec in zip(fgraph.outputs, outputs + additional_outputs)
-            if not spec.borrow
-        ]
-        if no_borrow:
-            self.linker = linker.accept(
-                fgraph,
-                no_recycling=infer_reuse_pattern(fgraph, no_borrow),
-                profile=profile,
-            )
-        else:
-            self.linker = linker.accept(fgraph, profile=profile)
-
-        if hasattr(linker, "accept_var_updates"):
-            # hacky thing so VMLinker knows about updates
-            self.linker.accept_var_updates(fgraph_updated_vars(fgraph, inputs))
-        fgraph.name = name
-        self.indices = indices
-        self.inputs = inputs
-        self.expanded_inputs = inputs
-        self.outputs = outputs
-        self.unpack_single = unpack_single
-        self.return_none = return_none
-        self.accept_inplace = accept_inplace
-        self.function_builder = function_builder
-        self.on_unused_input = on_unused_input  # Used for the pickling/copy
-        self.output_keys = output_keys
-        self.name = name
-
-        self.required = [(i.value is None) for i in self.inputs]
-        self.refeed = [
-            (
-                i.value is not None
-                and not isinstance(i.value, Container)
-                and i.update is None
-            )
-            for i in self.inputs
-        ]
-
-    def _check_unused_inputs(self, inputs, outputs, on_unused_input):
+    @staticmethod
+    def check_unused_inputs(inputs, outputs, on_unused_input):
         if on_unused_input is None:
             on_unused_input = config.on_unused_input
 
@@ -1589,6 +1380,174 @@ class FunctionMaker:
                         "Valid values are 'raise', 'warn', and 'ignore'."
                     )
 
+    def __init__(
+        self,
+        inputs,
+        outputs,
+        mode=None,
+        accept_inplace=False,
+        function_builder=Function,
+        profile=None,
+        on_unused_input=None,
+        fgraph=None,
+        output_keys=None,
+        name=None,
+    ):
+        # Save the provided mode, not the instantiated mode.
+        # The instantiated mode don't pickle and if we unpickle an Aesara
+        # function and it get re-compiled, we want the current optimizer to be
+        # used, not the optimizer when it was saved.
+        self.mode = mode
+        mode = aesara.compile.mode.get_mode(mode)
+
+        # Assert old way of working isn't used
+        if getattr(mode, "profile", None):
+            raise TypeError("profile passed via 'mode'. This isn't supported anymore")
+        self.profile = profile
+        if profile:
+            # This is very important:
+            # 1) We preload the cache here to not have its timing
+            #    included in optimization that compile function.
+            # 2) Do not refresh the cache here by default. It cause
+            #    too much execution time during testing as we compile
+            #    much more functions then the number of compile c
+            #    module.
+            aesara.link.c.basic.get_module_cache().refresh()
+        # Handle the case where inputs and/or outputs is a single
+        # Variable (not in a list)
+        unpack_single = False
+        return_none = False
+        if outputs is None:
+            return_none = True
+            outputs = []
+        if not isinstance(outputs, (list, tuple)):
+            unpack_single = True
+            outputs = [outputs]
+        if not isinstance(inputs, (list, tuple)):
+            inputs = [inputs]
+
+        # Wrap them in In or Out instances if needed.
+        inputs = [self.wrap_in(i) for i in inputs]
+        outputs = [self.wrap_out(o) for o in outputs]
+
+        # Check if some input variables are unused
+        self.check_unused_inputs(inputs, outputs, on_unused_input)
+
+        indices = [[input, None, [input]] for input in inputs]
+
+        if fgraph is None:
+            need_opt = True
+            # make the fgraph (copies the graph, creates NEW INPUT AND
+            # OUTPUT VARIABLES)
+            fgraph, additional_outputs = std_fgraph(inputs, outputs, accept_inplace)
+            fgraph.profile = profile
+        else:
+            # fgraph is already an optimized one
+            need_opt = False
+            updates = [spec.update for spec in inputs if spec.update]
+            additional_outputs = list(map(SymbolicOutput, updates))
+
+        self.fgraph = fgraph
+
+        optimizer, linker = mode.optimizer, copy.copy(mode.linker)
+        if need_opt:
+            # Why we add stack on node when it get done in output var?
+            try:
+                start_optimizer = time.time()
+
+                # In case there is an error during optimization.
+                optimizer_profile = None
+                opt_time = None
+
+                with config.change_flags(
+                    compute_test_value=config.compute_test_value_opt,
+                    traceback__limit=config.traceback__compile_limit,
+                ):
+                    optimizer_profile = optimizer(fgraph)
+
+                    end_optimizer = time.time()
+                    opt_time = end_optimizer - start_optimizer
+                    _logger.debug(f"Optimizing took {opt_time:f} seconds")
+
+                    # Add deep copy to respect the memory interface
+                    insert_deepcopy(fgraph, inputs, outputs + additional_outputs)
+            finally:
+
+                # If the optimizer got interrupted
+                if opt_time is None:
+                    end_optimizer = time.time()
+                    opt_time = end_optimizer - start_optimizer
+
+                aesara.compile.profiling.total_graph_opt_time += opt_time
+
+                if profile:
+                    if optimizer_profile is None and hasattr(optimizer, "pre_profile"):
+                        optimizer_profile = optimizer.pre_profile
+
+                    profile.optimizer_time += opt_time
+
+                    if config.profile_optimizer:
+                        profile.optimizer_profile = (optimizer, optimizer_profile)
+                # IF False, if mean the profile for that function was
+                # explicitly disabled
+                elif config.profile_optimizer and profile is not False:
+                    warnings.warn(
+                        (
+                            "config.profile_optimizer requires config.profile to "
+                            " be set to True as well"
+                        ),
+                        stacklevel=3,
+                    )
+
+        if not hasattr(linker, "accept"):
+            raise ValueError(
+                "'linker' parameter of FunctionMaker should be "
+                f"a Linker with an accept method or one of {list(aesara.compile.mode.predefined_linkers.keys())}"
+            )
+
+        # the 'no_borrow' outputs are the ones for which that we can't
+        # return the internal storage pointer.
+        assert len(fgraph.outputs) == len(outputs + additional_outputs)
+        no_borrow = [
+            output
+            for output, spec in zip(fgraph.outputs, outputs + additional_outputs)
+            if not spec.borrow
+        ]
+        if no_borrow:
+            self.linker = linker.accept(
+                fgraph,
+                no_recycling=infer_reuse_pattern(fgraph, no_borrow),
+                profile=profile,
+            )
+        else:
+            self.linker = linker.accept(fgraph, profile=profile)
+
+        if hasattr(linker, "accept_var_updates"):
+            # hacky thing so VMLinker knows about updates
+            self.linker.accept_var_updates(fgraph_updated_vars(fgraph, inputs))
+        fgraph.name = name
+        self.indices = indices
+        self.inputs = inputs
+        self.expanded_inputs = inputs
+        self.outputs = outputs
+        self.unpack_single = unpack_single
+        self.return_none = return_none
+        self.accept_inplace = accept_inplace
+        self.function_builder = function_builder
+        self.on_unused_input = on_unused_input  # Used for the pickling/copy
+        self.output_keys = output_keys
+        self.name = name
+
+        self.required = [(i.value is None) for i in self.inputs]
+        self.refeed = [
+            (
+                i.value is not None
+                and not isinstance(i.value, Container)
+                and i.update is None
+            )
+            for i in self.inputs
+        ]
+
     def create(self, input_storage=None, trustme=False, storage_map=None):
         """
         Create a function.
@@ -1704,33 +1663,6 @@ class FunctionMaker:
         return fn
 
 
-def _constructor_FunctionMaker(kwargs):
-    # Needed for old pickle
-    # Old pickle have at least the problem that output_keys where not saved.
-    if config.unpickle_function:
-        if config.reoptimize_unpickled_function:
-            del kwargs["fgraph"]
-        return FunctionMaker(**kwargs)
-    else:
-        return None
-
-
-__checkers: List = []
-
-
-def check_equal(x, y):
-    for checker in __checkers:
-        try:
-            return checker(x, y)
-        except Exception:
-            continue
-    return x == y
-
-
-def register_checker(checker):
-    __checkers.insert(0, checker)
-
-
 def orig_function(
     inputs,
     outputs,
@@ -1740,7 +1672,7 @@ def orig_function(
     profile=None,
     on_unused_input=None,
     output_keys=None,
-):
+) -> Function:
     """
     Return a Function that will calculate the outputs from the inputs.
 
@@ -1763,35 +1695,21 @@ def orig_function(
     profile : None or ProfileStats instance
     on_unused_input : {'raise', 'warn', 'ignore', None}
         What to do if a variable in the 'inputs' list is not used in the graph.
-    output_keys :
+    output_keys
         If the outputs were provided to aesara.function as a list, then
         output_keys is None. Otherwise, if outputs were provided as a dict,
         output_keys is the sorted list of keys from the outputs.
-
-    Notes
-    -----
-    Currently, the library provides the following mode strings:
-
-    - FAST_RUN (default) (optimize without too much time)
-
-    - FAST_COMPILE (minimal optimization)
-
-    - DebugMode: verify many internal conditions that are normally assumed
-      (slow)
+    fgraph
+        An existing `FunctionGraph` to use instead of constructing a new one
+        from cloned `outputs`.
 
     """
 
-    # Every element of the input list will be upgraded to an `In` instance if
-    # necessary, using the rules implemented by the `convert_function_input`
-    # function.
-
-    # Similarly, every element of the output list will be upgraded to an `Out`
-    # instance if necessary:
-
     t1 = time.time()
     mode = aesara.compile.mode.get_mode(mode)
 
     inputs = list(map(convert_function_input, inputs))
+
     if outputs is not None:
         if isinstance(outputs, (list, tuple)):
             outputs = list(map(FunctionMaker.wrap_out, outputs))
@@ -1800,8 +1718,9 @@ def orig_function(
 
     defaults = [getattr(input, "value", None) for input in inputs]
 
-    if isinstance(mode, (list, tuple)):  # "mode comparison" semantics
-        raise Exception("We do not support the passing of multiple modes")
+    if isinstance(mode, (list, tuple)):
+        raise ValueError("We do not support the passing of multiple modes")
+
     fn = None
     try:
         Maker = getattr(mode, "function_maker", FunctionMaker)
