@@ -10,6 +10,7 @@ import numpy as np
 import aesara
 from aesara.configdefaults import config
 from aesara.graph.basic import Variable, io_toposort
+from aesara.graph.utils import InconsistencyError
 
 
 class AlreadyThere(Exception):
@@ -327,20 +328,10 @@ class Feature:
 
 class Bookkeeper(Feature):
     def on_attach(self, fgraph):
-        """
-        Called by FunctionGraph.attach_feature, the method that attaches
-        the feature to the FunctionGraph. Since this is called after the
-        FunctionGraph is initially populated, this is where you should
-        run checks on the initial contents of the FunctionGraph.
-        """
         for node in io_toposort(fgraph.inputs, fgraph.outputs):
             self.on_import(fgraph, node, "on_attach")
 
     def on_detach(self, fgraph):
-        """
-        Should remove any dynamically added functionality
-        that it installed into the function_graph
-        """
         for node in io_toposort(fgraph.inputs, fgraph.outputs):
             self.on_prune(fgraph, node, "Bookkeeper.detach")
 
@@ -599,7 +590,9 @@ class ReplaceValidate(History, Validator):
             raise
 
         if verbose:
-            print(f"optimizer: rewrite {reason} replaces {r} with {new_r}")
+            print(
+                f"optimizer: rewrite {reason} replaces {r} of {r.owner} with {new_r} of {new_r.owner}"
+            )
 
         # The return is needed by replace_all_validate_remove
         return chk
@@ -639,9 +632,7 @@ class ReplaceValidate(History, Validator):
     def validate(self, fgraph):
         if self.fail_validate:
             self.fail_validate = False
-            raise aesara.graph.fg.InconsistencyError(
-                "Trying to reintroduce a removed node"
-            )
+            raise InconsistencyError("Trying to reintroduce a removed node")
 
 
 class NodeFinder(Bookkeeper):
@@ -737,18 +728,6 @@ class PrintListener(Feature):
             print(f"-- changing ({node}.inputs[{i}]) from {r} to {new_r}")
 
 
-class PreserveNames(Feature):
-    """
-    This preserve some variables names during optimization.
-
-    Deprecated. We need to keep it to allow unpickling.
-    """
-
-    def on_change_input(self, fgraph, node, i, r, new_r, reason=None):
-        if r.name is not None and new_r.name is None:
-            new_r.name = r.name
-
-
 class PreserveVariableAttributes(Feature):
     """
     This preserve some variables attributes and tag during optimization.
@@ -787,7 +766,7 @@ class NoOutputFromInplace(Feature):
             op = node.op
             out_idx = node.outputs.index(out)
             if out_idx in op.destroy_map:
-                raise aesara.graph.fg.InconsistencyError(
+                raise InconsistencyError(
                     "A function graph Feature has requested that outputs of the graph "
                     "be prevented from being the result of in-place "
                     f"operations. This has prevented the output {out} from "
