@@ -39,7 +39,6 @@ from aesara.tensor.extra_ops import (
     Bartlett,
     BroadcastTo,
     CumOp,
-    DiffOp,
     FillDiagonal,
     FillDiagonalOffset,
     RavelMultiIndex,
@@ -938,17 +937,6 @@ def jax_funcify_CumOp(op, **kwargs):
     return cumop
 
 
-@jax_funcify.register(DiffOp)
-def jax_funcify_DiffOp(op, **kwargs):
-    n = op.n
-    axis = op.axis
-
-    def diffop(x, n=n, axis=axis):
-        return jnp.diff(x, n=n, axis=axis)
-
-    return diffop
-
-
 @jax_funcify.register(Repeat)
 def jax_funcify_Repeat(op, **kwargs):
     axis = op.axis
