@@ -2,6 +2,7 @@ import operator
 import warnings
 from contextlib import contextmanager
 from functools import singledispatch
+from textwrap import dedent
 
 import numba
 import numba.np.unsafe.ndarray as numba_ndarray
@@ -11,11 +12,12 @@ import scipy.special
 from llvmlite.llvmpy.core import Type as llvm_Type
 from numba import types
 from numba.core.errors import TypingError
+from numba.cpython.unsafe.tuple import tuple_setitem  # noqa: F401
 from numba.extending import box
 
 from aesara import config
 from aesara.compile.ops import DeepCopyOp
-from aesara.graph.basic import Apply
+from aesara.graph.basic import Apply, NoParams
 from aesara.graph.fg import FunctionGraph
 from aesara.graph.type import Type
 from aesara.ifelse import IfElse
@@ -24,7 +26,7 @@ from aesara.link.utils import (
     fgraph_to_python,
     unique_name_generator,
 )
-from aesara.scalar.basic import Scalar
+from aesara.scalar.basic import ScalarType
 from aesara.scalar.math import Softplus
 from aesara.tensor.blas import BatchedDot
 from aesara.tensor.math import Dot
@@ -39,7 +41,7 @@ from aesara.tensor.subtensor import (
     Subtensor,
 )
 from aesara.tensor.type import TensorType
-from aesara.tensor.type_other import MakeSlice
+from aesara.tensor.type_other import MakeSlice, NoneConst
 
 
 def numba_njit(*args, **kwargs):
@@ -85,7 +87,7 @@ def get_numba_type(
         ):
             return numba_dtype
         return numba.types.Array(numba_dtype, aesara_type.ndim, layout)
-    elif isinstance(aesara_type, Scalar):
+    elif isinstance(aesara_type, ScalarType):
         dtype = np.dtype(aesara_type.dtype)
         numba_dtype = numba.from_dtype(dtype)
         return numba_dtype
@@ -329,16 +331,41 @@ def numba_funcify(op, node=None, storage_map=None, **kwargs):
     else:
         ret_sig = get_numba_type(node.outputs[0].type)
 
+    output_types = tuple(out.type for out in node.outputs)
+    params = node.run_params()
+
+    if params is not NoParams:
+        params_val = dict(node.params_type.filter(params))
+
+        def py_perform(inputs):
+            outputs = [[None] for i in range(n_outputs)]
+            op.perform(node, inputs, outputs, params_val)
+            return outputs
+
+    else:
+
+        def py_perform(inputs):
+            outputs = [[None] for i in range(n_outputs)]
+            op.perform(node, inputs, outputs)
+            return outputs
+
+    if n_outputs == 1:
+
+        def py_perform_return(inputs):
+            return output_types[0].filter(py_perform(inputs)[0][0])
+
+    else:
+
+        def py_perform_return(inputs):
+            return tuple(
+                out_type.filter(out[0])
+                for out_type, out in zip(output_types, py_perform(inputs))
+            )
+
     @numba_njit
     def perform(*inputs):
         with numba.objmode(ret=ret_sig):
-            outputs = [[None] for i in range(n_outputs)]
-            op.perform(node, inputs, outputs)
-            outputs = tuple([o[0] for o in outputs])
-            if n_outputs == 1:
-                ret = outputs[0]
-            else:
-                ret = outputs
+            ret = py_perform_return(inputs)
         return ret
 
     return perform
@@ -583,13 +610,28 @@ def numba_funcify_Reshape(op, **kwargs):
 
 
 @numba_funcify.register(SpecifyShape)
-def numba_funcify_SpecifyShape(op, **kwargs):
-    @numba_njit
-    def specifyshape(x, shape):
-        assert np.array_equal(x.shape, shape)
-        return x
+def numba_funcify_SpecifyShape(op, node, **kwargs):
+    shape_inputs = node.inputs[1:]
+    shape_input_names = ["shape_" + str(i) for i in range(len(shape_inputs))]
 
-    return specifyshape
+    func_conditions = [
+        f"assert x.shape[{i}] == {shape_input_names}"
+        for i, (shape_input, shape_input_names) in enumerate(
+            zip(shape_inputs, shape_input_names)
+        )
+        if shape_input is not NoneConst
+    ]
+
+    func = dedent(
+        f"""
+        def specify_shape(x, {create_arg_string(shape_input_names)}):
+            {"; ".join(func_conditions)}
+            return x
+        """
+    )
+
+    specify_shape = compile_function_src(func, "specify_shape", globals())
+    return numba_njit(specify_shape)
 
 
 def int_to_float_fn(inputs, out_dtype):
