@@ -1,19 +1,17 @@
-import logging
-from collections import OrderedDict
+import warnings
 
 import numpy as np
 
 import aesara.tensor as at
-from aesara.compile import SharedVariable
 from aesara.compile.function.pfunc import construct_pfunc_ins_and_outs
+from aesara.compile.sharedvalue import SharedVariable, collect_new_shareds
 from aesara.configdefaults import config
 from aesara.graph.basic import Constant, Variable, clone_replace, graph_inputs
-from aesara.graph.fg import MissingInputError
 from aesara.graph.op import get_test_value
-from aesara.graph.utils import TestValueError
-from aesara.scan import utils
+from aesara.graph.utils import MissingInputError, TestValueError
 from aesara.scan.op import Scan, ScanInfo
-from aesara.scan.utils import safe_new, traverse
+from aesara.scan.utils import expand_empty, safe_new, until
+from aesara.tensor.basic import get_scalar_constant_value
 from aesara.tensor.exceptions import NotScalarConstantError
 from aesara.tensor.math import minimum
 from aesara.tensor.shape import shape_padleft
@@ -21,7 +19,144 @@ from aesara.tensor.type import TensorType, integer_dtypes
 from aesara.updates import OrderedUpdates
 
 
-_logger = logging.getLogger("aesara.scan.basic")
+def get_updates_and_outputs(ls):
+    """Recognize and order the updates, outputs, and stopping condition for a `Scan`.
+
+    WRITEME: what is the type of ls? how is it formatted?  if it's not in the
+    predefined order already, how does this function know how to put it in that
+    order?
+
+    """
+
+    def is_outputs(elem):
+        if isinstance(elem, (list, tuple)) and all(
+            [isinstance(x, Variable) for x in elem]
+        ):
+            return True
+        if isinstance(elem, Variable):
+            return True
+        return False
+
+    def is_updates(elem):
+        if isinstance(elem, dict):
+            # Make sure the updates will be applied in a deterministic order
+            return True
+        # Dictionaries can be given as lists of tuples
+        if isinstance(elem, (list, tuple)) and all(
+            [isinstance(x, (list, tuple)) and len(x) == 2 for x in elem]
+        ):
+            return True
+        return False
+
+    def is_condition(elem):
+        return isinstance(elem, until)
+
+    def _list(x):
+        if isinstance(x, (list, tuple)):
+            return list(x)
+        else:
+            return [x]
+
+    def _filter(x):
+        """
+        Ensure `x` is made only of allowed data types.
+
+        Return True iff `x` is made only of lists, tuples, dictionaries, Aesara
+        variables or `aesara.scan.utils.until` objects.
+
+        """
+        # Is `x` a container we can iterate on?
+        iter_on = None
+        if isinstance(x, list) or isinstance(x, tuple):
+            iter_on = x
+        elif isinstance(x, dict):
+            iter_on = x.items()
+        if iter_on is not None:
+            return all(_filter(y) for y in iter_on)
+        else:
+            return isinstance(x, Variable) or isinstance(x, until)
+
+    if not _filter(ls):
+        raise ValueError(
+            "The return value of your scan lambda expression may only be "
+            "made of lists, tuples, or dictionaries containing Aesara "
+            "variables (or `aesara.scan.utils.until` objects for "
+            "conditions). In particular if you need to use constant "
+            "values, you can use `tensor.constant` to turn them into "
+            "Aesara variables."
+        )
+
+    if is_outputs(ls):
+        return None, _list(ls), dict()
+    if is_updates(ls):
+        return None, [], dict(ls)
+    error_msg = (
+        f"Scan cannot parse the return value of your lambda expression, which is: {ls}"
+    )
+    if not isinstance(ls, (list, tuple)):
+        raise ValueError(error_msg)
+    ls = list(ls)
+    deprecation_msg = (
+        "The return value of the lambda function"
+        " has been restricted. you have to always return first the"
+        " outputs (if any), afterwards the updates (if any) and"
+        " at the end the conclusion"
+    )
+    if len(ls) == 2:
+        if is_outputs(ls[0]):
+            if is_updates(ls[1]):
+                return (None, _list(ls[0]), dict(ls[1]))
+            elif is_condition(ls[1]):
+                return (ls[1].condition, _list(ls[0]), dict())
+            else:
+                raise ValueError(error_msg)
+        elif is_updates(ls[0]):
+            if is_outputs(ls[1]):
+                raise ValueError(deprecation_msg)
+            elif is_condition(ls[1]):
+                return (ls[1].condition, [], dict(ls[0]))
+            else:
+                raise ValueError(error_msg)
+        else:
+            raise ValueError(error_msg)
+    elif len(ls) == 3:
+        if is_outputs(ls[0]):
+            if is_updates(ls[1]):
+                if is_condition(ls[2]):
+                    return (ls[2].condition, _list(ls[0]), dict(ls[1]))
+                else:
+                    raise ValueError(error_msg)
+            else:
+                raise ValueError(error_msg)
+        else:
+            raise ValueError(error_msg)
+    else:
+        raise ValueError(error_msg)
+
+
+def isNaN_or_Inf_or_None(x):
+    isNone = x is None
+    try:
+        isNaN = np.isnan(x)
+        isInf = np.isinf(x)
+        isStr = isinstance(x, str)
+    except Exception:
+        isNaN = False
+        isInf = False
+        isStr = False
+    if not isNaN and not isInf:
+        try:
+            val = get_scalar_constant_value(x)
+            isInf = np.isinf(val)
+            isNaN = np.isnan(val)
+        except Exception:
+            isNaN = False
+            isInf = False
+    if isinstance(x, Constant) and isinstance(x.data, str):
+        isStr = True
+    else:
+        isStr = False
+    return isNone or isNaN or isInf or isStr
 
 
 def scan(
@@ -278,10 +413,6 @@ def scan(
         allocations are freed at the end of all iterations; this is what the
         flag `aesara.config.allow_gc` means.
 
-        If you use pre-allocation and this `Scan` is on GPU, the speed up from
-        `allow_gc` is small. If you are missing memory, disabling `allow_gc`
-        could help you run graph that request much memory.
-
     strict
         If ``True``, all the shared variables used in `fn` must be provided as a
         part of `non_sequences` or `sequences`.
@@ -356,11 +487,11 @@ def scan(
     n_seqs = len(seqs)
     n_outs = len(outs_info)
 
-    return_steps = OrderedDict()
+    return_steps = {}
     # wrap sequences in a dictionary if they are not already dictionaries
     for i in range(n_seqs):
         if not isinstance(seqs[i], dict):
-            seqs[i] = OrderedDict([("input", seqs[i]), ("taps", [0])])
+            seqs[i] = dict([("input", seqs[i]), ("taps", [0])])
         elif seqs[i].get("taps", None) is not None:
             seqs[i]["taps"] = wrap_into_list(seqs[i]["taps"])
         elif seqs[i].get("taps", None) is None:
@@ -382,7 +513,7 @@ def scan(
 
             if not isinstance(outs_info[i], dict):
                 # by default any output has a tap value of -1
-                outs_info[i] = OrderedDict([("initial", outs_info[i]), ("taps", [-1])])
+                outs_info[i] = dict([("initial", outs_info[i]), ("taps", [-1])])
             elif (
                 outs_info[i].get("initial", None) is None
                 and outs_info[i].get("taps", None) is not None
@@ -400,7 +531,7 @@ def scan(
                 # ^ initial state but taps not provided
                 if "taps" in outs_info[i]:
                     # ^ explicitly provided a None for taps
-                    _logger.warning(
+                    warnings.warn(
                         f"Output {getattr(outs_info[i]['initial'], 'name', 'None')} (index {i}) has a initial "
                         "state but taps is explicitly set to None ",
                     )
@@ -422,7 +553,7 @@ def scan(
         else:
             # if a None is provided as the output info we replace it
             # with an empty OrdereDict() to simplify handling
-            outs_info[i] = OrderedDict()
+            outs_info[i] = {}
 
     ##
     # Step 2. Generate inputs and outputs of the inner functions
@@ -477,12 +608,12 @@ def scan(
                         if config.compute_test_value != "ignore":
                             # No need to print a warning or raise an error now,
                             # it will be done when fn will be called.
-                            _logger.warning(
+                            warnings.warn(
                                 (
                                     "Cannot compute test value for "
                                     "the inner function of scan, input value "
-                                    "missing {}"
-                                ).format(_seq_val_slice)
+                                    f"missing {_seq_val_slice}"
+                                )
                             )
 
                 # Add names to slices for debugging and pretty printing ..
@@ -525,7 +656,7 @@ def scan(
     for seq in scan_seqs:
         lengths_vec.append(seq.shape[0])
 
-    if not utils.isNaN_or_Inf_or_None(n_steps):
+    if not isNaN_or_Inf_or_None(n_steps):
         # ^ N_steps should also be considered
         lengths_vec.append(at.as_tensor(n_steps))
 
@@ -540,7 +671,7 @@ def scan(
 
     # If the user has provided the number of steps, do that regardless ( and
     # raise an error if the sequences are not long enough )
-    if utils.isNaN_or_Inf_or_None(n_steps):
+    if isNaN_or_Inf_or_None(n_steps):
         actual_n_steps = lengths_vec[0]
         for contestant in lengths_vec[1:]:
             actual_n_steps = minimum(actual_n_steps, contestant)
@@ -569,7 +700,7 @@ def scan(
     mit_sot_inner_inputs = []
     mit_sot_inner_slices = []
     mit_sot_inner_outputs = []
-    mit_sot_return_steps = OrderedDict()
+    mit_sot_return_steps = {}
     mit_sot_tap_array = []
     mit_sot_rightOrder = []
 
@@ -578,7 +709,7 @@ def scan(
     sit_sot_inner_inputs = []
     sit_sot_inner_slices = []
     sit_sot_inner_outputs = []
-    sit_sot_return_steps = OrderedDict()
+    sit_sot_return_steps = {}
     sit_sot_rightOrder = []
 
     # go through outputs picking up time slices as needed
@@ -606,11 +737,11 @@ def scan(
                     arg.tag.test_value = get_test_value(actual_arg)
                 except TestValueError:
                     if config.compute_test_value != "ignore":
-                        _logger.warning(
+                        warnings.warn(
                             (
                                 "Cannot compute test value for the "
-                                "inner function of scan, test value missing: {}"
-                            ).format(actual_arg)
+                                f"inner function of scan, test value missing: {actual_arg}"
+                            )
                         )
 
             if getattr(init_out["initial"], "name", None) is not None:
@@ -620,7 +751,7 @@ def scan(
             # the initial state over. We do this using the expand function
             # defined in scan utils
             sit_sot_scan_inputs.append(
-                utils.expand_empty(
+                expand_empty(
                     at.unbroadcast(shape_padleft(actual_arg), 0),
                     actual_n_steps,
                 )
@@ -644,7 +775,7 @@ def scan(
             mit_sot_tap_array.append(init_out["taps"])
             # Sequence
             mit_sot_scan_inputs.append(
-                utils.expand_empty(init_out["initial"][:mintap], actual_n_steps)
+                expand_empty(init_out["initial"][:mintap], actual_n_steps)
             )
 
             if i in return_steps:
@@ -664,12 +795,12 @@ def scan(
                         nw_slice.tag.test_value = get_test_value(_init_out_var_slice)
                     except TestValueError:
                         if config.compute_test_value != "ignore":
-                            _logger.warning(
+                            warnings.warn(
                                 (
                                     "Cannot compute test value for "
                                     "the inner function of scan, test value "
-                                    "missing: {}"
-                                ).format(_init_out_var_slice)
+                                    f"missing: {_init_out_var_slice}"
+                                )
                             )
 
                 # give it a name or debugging and pretty printing
@@ -730,7 +861,10 @@ def scan(
     # when we apply the lambda expression we get a mixture of update rules
     # and outputs that needs to be separated
 
-    condition, outputs, updates = utils.get_updates_and_outputs(fn(*args))
+    with collect_new_shareds() as new_shareds:
+        raw_inner_outputs = fn(*args)
+
+    condition, outputs, updates = get_updates_and_outputs(raw_inner_outputs)
     if condition is not None:
         as_while = True
     else:
@@ -769,9 +903,7 @@ def scan(
     if condition is not None:
         outputs.append(condition)
     fake_nonseqs = [x.type() for x in non_seqs]
-    fake_outputs = clone_replace(
-        outputs, replace=OrderedDict(zip(non_seqs, fake_nonseqs))
-    )
+    fake_outputs = clone_replace(outputs, replace=dict(zip(non_seqs, fake_nonseqs)))
     all_inputs = filter(
         lambda x: (
             isinstance(x, Variable)
@@ -825,7 +957,7 @@ def scan(
         n_outs = len(dummy_outputs)
         if as_while:
             n_outs = n_outs - 1
-        outs_info = [OrderedDict() for x in range(n_outs)]
+        outs_info = [{} for x in range(n_outs)]
 
     # Step 5.1 Outputs with taps different then -1
 
@@ -839,25 +971,51 @@ def scan(
             sit_sot_inner_outputs.append(outputs[i])
 
     # Step 5.3 Outputs that correspond to update rules of shared variables
-    givens = OrderedDict()
+    inner_replacements = {}
     n_shared_outs = 0
     shared_scan_inputs = []
     shared_inner_inputs = []
     shared_inner_outputs = []
     sit_sot_shared = []
+    no_update_shared_inputs = []
     for input in dummy_inputs:
-        if isinstance(input.variable, SharedVariable) and input.update:
+        if not isinstance(input.variable, SharedVariable):
+            continue
+
+        is_local = input.variable in new_shareds
+
+        # We only want to add shared variable updates that were either
+        # user-specified within the inner-function (e.g. by returning an update
+        # `dict`) or the `SharedVariable.default_update`s of a shared variable
+        # created in the inner-function.
+        if input.update and (is_local or input.variable in updates):
+            # We need to remove the `default_update`s on the shared
+            # variables created within the context of the loop function
+            # (e.g. via use of `RandomStream`); otherwise, they'll get
+            # picked up during compilation and produce errors when the
+            # updates include inner-graph variables.
+            # We also don't want to remove a default update that applies to
+            # the scope/context containing this `Scan`, so we only remove
+            # default updates on "local" variables.
+            if is_local and hasattr(input.variable, "default_update"):
+                del input.variable.default_update
+
             new_var = safe_new(input.variable)
+
             if getattr(input.variable, "name", None) is not None:
                 new_var.name = input.variable.name + "_copy"
+
+            inner_replacements[input.variable] = new_var
+
             if isinstance(new_var.type, TensorType):
                 sit_sot_inner_inputs.append(new_var)
                 sit_sot_scan_inputs.append(
-                    utils.expand_empty(
+                    expand_empty(
                         at.unbroadcast(shape_padleft(input.variable), 0),
                         actual_n_steps,
                     )
                 )
+
                 tensor_update = at.as_tensor_variable(input.update)
                 sit_sot_inner_outputs.append(tensor_update)
                 # Note that `pos` is not a negative index. The sign of `pos` is used
@@ -869,21 +1027,21 @@ def scan(
                 # refers to the update rule with index `-1 - pos`.
                 sit_sot_rightOrder.append(-1 - len(sit_sot_shared))
                 sit_sot_shared.append(input.variable)
-                givens[input.variable] = new_var
 
             else:
                 shared_inner_inputs.append(new_var)
                 shared_scan_inputs.append(input.variable)
                 shared_inner_outputs.append(input.update)
-                givens[input.variable] = new_var
                 n_shared_outs += 1
+        else:
+            no_update_shared_inputs.append(input)
 
     n_sit_sot = len(sit_sot_inner_inputs)
 
     # Step 5.4 Outputs with no taps used in the input
     n_nit_sot = 0
     nit_sot_inner_outputs = []
-    nit_sot_return_steps = OrderedDict()
+    nit_sot_return_steps = {}
     nit_sot_rightOrder = []
     for i, out in enumerate(outs_info):
         if "taps" not in out:
@@ -910,45 +1068,34 @@ def scan(
         if (not isinstance(arg, SharedVariable) and not isinstance(arg, Constant))
     ]
 
-    givens.update(OrderedDict(zip(other_scan_args, other_inner_args)))
+    inner_replacements.update(dict(zip(other_scan_args, other_inner_args)))
 
     if strict:
         non_seqs_set = set(non_sequences if non_sequences is not None else [])
 
         other_shared_scan_args = [
             arg.variable
-            for arg in dummy_inputs
-            if (
-                isinstance(arg.variable, SharedVariable)
-                and not arg.update
-                and arg.variable in non_seqs_set
-            )
+            for arg in no_update_shared_inputs
+            if arg.variable in non_seqs_set
         ]
         other_shared_inner_args = [
             safe_new(arg.variable, "_copy")
-            for arg in dummy_inputs
-            if (
-                isinstance(arg.variable, SharedVariable)
-                and not arg.update
-                and arg.variable in non_seqs_set
-            )
+            for arg in no_update_shared_inputs
+            if arg.variable in non_seqs_set
         ]
     else:
-        other_shared_scan_args = [
-            arg.variable
-            for arg in dummy_inputs
-            if (isinstance(arg.variable, SharedVariable) and not arg.update)
-        ]
+        other_shared_scan_args = [arg.variable for arg in no_update_shared_inputs]
         other_shared_inner_args = [
-            safe_new(arg.variable, "_copy")
-            for arg in dummy_inputs
-            if (isinstance(arg.variable, SharedVariable) and not arg.update)
+            safe_new(arg.variable, "_copy") for arg in no_update_shared_inputs
         ]
-    givens.update(OrderedDict(zip(other_shared_scan_args, other_shared_inner_args)))
+
+    inner_replacements.update(
+        dict(zip(other_shared_scan_args, other_shared_inner_args))
+    )
 
     ##
     # Step 6. Re-order the outputs and clone them replacing things
-    # using the givens
+    # using `inner_replacements`
     ##
     inner_inputs = (
         inner_seqs
@@ -969,31 +1116,8 @@ def scan(
     )
     if condition is not None:
         inner_outs.append(condition)
-    # gpuarray is imported here, instead of being imported on top of
-    # the file because that would force on the user some dependencies that we
-    # might do not want to. Currently we are working on removing the
-    # dependencies on sandbox code completely.
-    from aesara import gpuarray
 
-    if gpuarray.pygpu_activated:
-        # very often we end up in this situation when we want to
-        # replace w with w_copy, where w is a GPU variable
-        # and w_copy is TensorType. This is caused because shared
-        # variables are put on GPU right away >:| ,
-        new_givens = OrderedDict()
-
-        for w, w_copy in givens.items():
-            if isinstance(w.type, gpuarray.GpuArrayType) and isinstance(
-                w_copy.type, TensorType
-            ):
-                for o in inner_outs:
-                    new_givens = traverse(o, w, w_copy, new_givens)
-            else:
-                new_givens[w] = w_copy
-    else:
-        new_givens = givens
-
-    new_outs = clone_replace(inner_outs, replace=new_givens)
+    new_outs = clone_replace(inner_outs, replace=inner_replacements)
 
     ##
     # Step 7. Create the Scan Op
@@ -1024,7 +1148,6 @@ def scan(
         mode=mode,
         truncate_gradient=truncate_gradient,
         name=name,
-        gpua=False,
         as_while=as_while,
         profile=profile,
         allow_gc=allow_gc,
