@@ -15,7 +15,6 @@ There are four kinds of BLAS Ops in Aesara:
     - Python implementations (this file)
     - SciPy-based (blas_scipy)
     - C-based (blas_c)
-    - GPU-based (aesara.gpuarray)
 
 Notes
 -----
@@ -147,27 +146,32 @@ from aesara.compile.mode import optdb
 from aesara.configdefaults import config
 from aesara.graph.basic import Apply, view_roots
 from aesara.graph.features import ReplacementDidNotRemoveError, ReplaceValidate
-from aesara.graph.fg import InconsistencyError
-from aesara.graph.op import COp, Op
+from aesara.graph.op import Op
 from aesara.graph.opt import (
     EquilibriumOptimizer,
     GlobalOptimizer,
+    copy_stack_trace,
     in2out,
-    inherit_stack_trace,
     local_optimizer,
 )
 from aesara.graph.optdb import SequenceDB
-from aesara.graph.params_type import ParamsType
-from aesara.graph.utils import MethodNotDefined, TestValueError
+from aesara.graph.utils import InconsistencyError, MethodNotDefined, TestValueError
+from aesara.link.c.op import COp
+from aesara.link.c.params_type import ParamsType
 from aesara.printing import FunctionPrinter, debugprint, pprint
 from aesara.scalar import bool as bool_t
-from aesara.tensor import basic as aet
+from aesara.tensor import basic as at
 from aesara.tensor.basic_opt import local_dimshuffle_lift
 from aesara.tensor.blas_headers import blas_header_text, blas_header_version
 from aesara.tensor.elemwise import DimShuffle, Elemwise
 from aesara.tensor.exceptions import NotScalarConstantError
 from aesara.tensor.math import Dot, add, mul, neg, sub
-from aesara.tensor.type import integer_dtypes, tensor, values_eq_approx_remove_inf_nan
+from aesara.tensor.type import (
+    DenseTensorType,
+    integer_dtypes,
+    tensor,
+    values_eq_approx_remove_inf_nan,
+)
 from aesara.utils import memoize
 
 
@@ -249,11 +253,11 @@ class Gemv(Op):
             return "%s{no_inplace}" % self.__class__.__name__
 
     def make_node(self, y, alpha, A, x, beta):
-        y = aet.as_tensor_variable(y)
-        x = aet.as_tensor_variable(x)
-        A = aet.as_tensor_variable(A)
-        alpha = aet.as_tensor_variable(alpha)
-        beta = aet.as_tensor_variable(beta)
+        y = at.as_tensor_variable(y)
+        x = at.as_tensor_variable(x)
+        A = at.as_tensor_variable(A)
+        alpha = at.as_tensor_variable(alpha)
+        beta = at.as_tensor_variable(beta)
         if y.dtype != A.dtype or y.dtype != x.dtype:
             raise TypeError(
                 "Gemv requires matching dtypes", (y.dtype, A.dtype, x.dtype)
@@ -264,7 +268,13 @@ class Gemv(Op):
             raise TypeError("gemv requires vector for x", x.type)
         if y.ndim != 1:
             raise TypeError("gemv requires vector for y", y.type)
-        return Apply(self, [y, alpha, A, x, beta], [y.type()])
+
+        inputs = [y, alpha, A, x, beta]
+
+        if any(not isinstance(i.type, DenseTensorType) for i in inputs):
+            raise NotImplementedError("Only dense tensor types are supported")
+
+        return Apply(self, inputs, [y.type()])
 
     def perform(self, node, inputs, out_storage, params=None):
         y, alpha, A, x, beta = inputs
@@ -342,10 +352,10 @@ class Ger(Op):
             return "%s{non-destructive}" % self.__class__.__name__
 
     def make_node(self, A, alpha, x, y):
-        A = aet.as_tensor_variable(A)
-        y = aet.as_tensor_variable(y)
-        x = aet.as_tensor_variable(x)
-        alpha = aet.as_tensor_variable(alpha)
+        A = at.as_tensor_variable(A)
+        y = at.as_tensor_variable(y)
+        x = at.as_tensor_variable(x)
+        alpha = at.as_tensor_variable(alpha)
         if not (A.dtype == x.dtype == y.dtype == alpha.dtype):
             raise TypeError(
                 "ger requires matching dtypes", (A.dtype, alpha.dtype, x.dtype, y.dtype)
@@ -361,7 +371,12 @@ class Ger(Op):
 
         if x.dtype not in ("float32", "float64", "complex64", "complex128"):
             raise TypeError("only float and complex types supported", x.dtype)
-        return Apply(self, [A, alpha, x, y], [A.type()])
+
+        inputs = [A, alpha, x, y]
+        if any(not isinstance(i.type, DenseTensorType) for i in inputs):
+            raise NotImplementedError("Only dense tensor types are supported")
+
+        return Apply(self, inputs, [A.type()])
 
     def perform(self, node, inp, out, params=None):
         cA, calpha, cx, cy = inp
@@ -458,7 +473,7 @@ def _ldflags(ldflags_str, libs, flags, libs_dir, include_dir):
         for d in dirs:
             for f in os.listdir(d.strip('"')):
                 if f.endswith(".so") or f.endswith(".dylib") or f.endswith(".dll"):
-                    if any([f.find(ll) >= 0 for ll in l]):
+                    if any(f.find(ll) >= 0 for ll in l):
                         found_dyn = True
         if not found_dyn and dirs:
             _logger.warning(
@@ -491,7 +506,7 @@ def _ldflags(ldflags_str, libs, flags, libs_dir, include_dir):
             rval.append(t[2:])
         elif libs and t1 == "l":  # example -lmkl
             rval.append(t[2:])
-        elif flags and t1 not in ["L", "I", "l"]:  # example -openmp
+        elif flags and t1 not in ("L", "I", "l"):  # example -openmp
             rval.append(t)
         elif flags and t1 == "L":
             # to find it when we load the compiled op if the env of the
@@ -898,7 +913,11 @@ class Gemm(GemmRelated):
         return rval
 
     def make_node(self, *inputs):
-        inputs = list(map(aet.as_tensor_variable, inputs))
+        inputs = list(map(at.as_tensor_variable, inputs))
+
+        if any(not isinstance(i.type, DenseTensorType) for i in inputs):
+            raise NotImplementedError("Only dense tensor types are supported")
+
         if len(inputs) != 5:
             raise TypeError(
                 f"Wrong number of inputs for {self} (expected 5, got {len(inputs)})"
@@ -1112,7 +1131,7 @@ def res_is_a(fgraph, var, op, maxclients=None):
 
 
 def _as_scalar(res, dtype=None):
-    """Return ``None`` or a `TensorVariable` whose type is in `float_scalar_types`"""
+    """Return ``None`` or a `TensorVariable` of float type"""
     if dtype is None:
         dtype = config.floatX
     if np.all(res.type.broadcastable):
@@ -1130,7 +1149,7 @@ def _as_scalar(res, dtype=None):
             # as the cast of the scalar can be done before or after the dot22
             # and this will give the same result.
             if aesara.scalar.upcast(res.dtype, dtype) == dtype:
-                return aet.cast(rval, dtype)
+                return at.cast(rval, dtype)
             else:
                 return None
 
@@ -1343,9 +1362,9 @@ def _gemm_from_factored_list(fgraph, lst):
         # sM can be a tuple of 2 elements or an Aesara variable.
         if isinstance(sM, tuple):
             sm0, sm1 = sM
-            sm0 = aet.as_tensor_variable(sm0)
+            sm0 = at.as_tensor_variable(sm0)
             if aesara.scalar.upcast(sm0.dtype, sm1.dtype) == sm1.dtype:
-                lst2.append((aet.cast(sm0, sm1.dtype), sM[1]))
+                lst2.append((at.cast(sm0, sm1.dtype), sM[1]))
 
     lst = lst2
 
@@ -1367,7 +1386,7 @@ def _gemm_from_factored_list(fgraph, lst):
         for j in range(i + 1, len(lst)):
             s_j, M_j = lst[j]
 
-            if M_i.type != M_j.type:
+            if not M_j.type.in_same_class(M_i.type):
                 continue
 
             # print 'TRYING', (s_i, M_i, s_j, M_j)
@@ -1393,11 +1412,11 @@ def _gemm_from_factored_list(fgraph, lst):
 
 def _gemm_from_node2(fgraph, node):
     """
-    :todo: In many expressions, there are many ways to turn it into a
-        gemm.  For example dot(a,b) + c + d.  This function should
-        return all of them, so that if one version of gemm causes a
-        cycle in the graph, then another application of gemm can be
-        tried.
+
+    TODO: In many expressions, there are many ways to turn it into a
+    gemm.  For example dot(a,b) + c + d.  This function should return all
+    of them, so that if one version of gemm causes a cycle in the graph, then
+    another application of gemm can be tried.
 
     """
     lst = []
@@ -1405,7 +1424,6 @@ def _gemm_from_node2(fgraph, node):
     _gemm_canonicalize(fgraph, node.outputs[0], 1.0, lst, 0)
     t1 = time.time()
 
-    # print "GEMM CANON", lst
     if len(lst) > 1:
         lst = _factor_canonicalized(lst)
         t2 = time.time()
@@ -1421,7 +1439,7 @@ def _gemm_from_node2(fgraph, node):
         # http://groups.google.com/group/theano-dev/browse_thread/thread/a3096c82856e3ad5,
         # but never made it into a trac ticket.
 
-        if rval and (rval[0][0].type == node.outputs[0].type):
+        if rval and rval[0][0].type.in_same_class(node.outputs[0].type):
             return rval, t1 - t0, t2 - t1, t3 - t2
 
     return None, t1 - t0, 0, 0
@@ -1579,8 +1597,12 @@ class Dot22(GemmRelated):
     check_input = False
 
     def make_node(self, x, y):
-        x = aet.as_tensor_variable(x)
-        y = aet.as_tensor_variable(y)
+        x = at.as_tensor_variable(x)
+        y = at.as_tensor_variable(y)
+
+        if any(not isinstance(i.type, DenseTensorType) for i in (x, y)):
+            raise NotImplementedError("Only dense tensor types are supported")
+
         dtypes = ("float16", "float32", "float64", "complex64", "complex128")
         if x.type.ndim != 2 or x.type.dtype not in dtypes:
             raise TypeError(x)
@@ -1666,22 +1688,28 @@ def local_dot_to_dot22(fgraph, node):
     if not isinstance(node.op, Dot):
         return
 
+    if any(not isinstance(i.type, DenseTensorType) for i in node.inputs):
+        return False
+
     x, y = node.inputs
     if y.type.dtype != x.type.dtype:
         # TODO: upcast one so the types match
         _logger.info(f"Not optimizing dot with inputs {x} {y} {x.type} {y.type}")
         return
 
-    if y.type.dtype in ["float16", "float32", "float64", "complex64", "complex128"]:
-        with inherit_stack_trace(node.outputs):
-            if x.ndim == 2 and y.ndim == 2:
-                return [_dot22(*node.inputs)]
-            if x.ndim == 2 and y.ndim == 1:
-                return [_dot22(x, y.dimshuffle(0, "x")).dimshuffle(0)]
-            if x.ndim == 1 and y.ndim == 2:
-                return [_dot22(x.dimshuffle("x", 0), y).dimshuffle(1)]
-            if x.ndim == 1 and y.ndim == 1:
-                return [_dot22(x.dimshuffle("x", 0), y.dimshuffle(0, "x")).dimshuffle()]
+    if y.type.dtype in ("float16", "float32", "float64", "complex64", "complex128"):
+        if x.ndim == 2 and y.ndim == 2:
+            new_out = [_dot22(*node.inputs)]
+        elif x.ndim == 2 and y.ndim == 1:
+            new_out = [_dot22(x, y.dimshuffle(0, "x")).dimshuffle(0)]
+        elif x.ndim == 1 and y.ndim == 2:
+            new_out = [_dot22(x.dimshuffle("x", 0), y).dimshuffle(1)]
+        elif x.ndim == 1 and y.ndim == 1:
+            new_out = [_dot22(x.dimshuffle("x", 0), y.dimshuffle(0, "x")).dimshuffle()]
+        else:
+            return
+        copy_stack_trace(node.outputs, new_out)
+        return new_out
 
     _logger.info(f"Not optimizing dot with inputs {x} {y} {x.type} {y.type}")
 
@@ -1689,22 +1717,25 @@ def local_dot_to_dot22(fgraph, node):
 @local_optimizer([gemm_no_inplace], inplace=True)
 def local_inplace_gemm(fgraph, node):
     if node.op == gemm_no_inplace:
-        with inherit_stack_trace(node.outputs):
-            return [gemm_inplace(*node.inputs)]
+        new_out = [gemm_inplace(*node.inputs)]
+        copy_stack_trace(node.outputs, new_out)
+        return new_out
 
 
 @local_optimizer([gemv_no_inplace], inplace=True)
 def local_inplace_gemv(fgraph, node):
     if node.op == gemv_no_inplace:
-        with inherit_stack_trace(node.outputs):
-            return [gemv_inplace(*node.inputs)]
+        new_out = [gemv_inplace(*node.inputs)]
+        copy_stack_trace(node.outputs, new_out)
+        return new_out
 
 
 @local_optimizer([ger], inplace=True)
 def local_inplace_ger(fgraph, node):
     if node.op == ger:
-        with inherit_stack_trace(node.outputs):
-            return [ger_destructive(*node.inputs)]
+        new_out = [ger_destructive(*node.inputs)]
+        copy_stack_trace(node.outputs, new_out)
+        return new_out
 
 
 @local_optimizer([gemm_no_inplace])
@@ -1712,13 +1743,16 @@ def local_gemm_to_gemv(fgraph, node):
     """GEMM acting on row or column matrices -> GEMV."""
     if node.op == gemm_no_inplace:
         z, a, x, y, b = node.inputs
-        with inherit_stack_trace(node.outputs):
-            if z.broadcastable == x.broadcastable == (True, False):
-                r = gemv_no_inplace(z.dimshuffle(1), a, y.T, x.dimshuffle(1), b)
-                return [r.dimshuffle("x", 0)]
-            if z.broadcastable == y.broadcastable == (False, True):
-                r = gemv_no_inplace(z.dimshuffle(0), a, x, y.dimshuffle(0), b)
-                return [r.dimshuffle(0, "x")]
+        if z.broadcastable == x.broadcastable == (True, False):
+            r = gemv_no_inplace(z.dimshuffle(1), a, y.T, x.dimshuffle(1), b)
+            new_out = [r.dimshuffle("x", 0)]
+        elif z.broadcastable == y.broadcastable == (False, True):
+            r = gemv_no_inplace(z.dimshuffle(0), a, x, y.dimshuffle(0), b)
+            new_out = [r.dimshuffle(0, "x")]
+        else:
+            return
+        copy_stack_trace(node.outputs, new_out)
+        return new_out
 
 
 @local_optimizer([gemm_no_inplace])
@@ -1727,27 +1761,28 @@ def local_gemm_to_ger(fgraph, node):
     if node.op == gemm_no_inplace:
         z, a, x, y, b = node.inputs
         if x.broadcastable[1] and y.broadcastable[0]:
-            with inherit_stack_trace(node.outputs):
-                # x and y are both vectors so this might qualifies for a GER
-                xv = x.dimshuffle(0)
-                yv = y.dimshuffle(1)
-                try:
-                    bval = aet.get_scalar_constant_value(b)
-                except NotScalarConstantError:
-                    # b isn't a constant, GEMM is doing useful pre-scaling
-                    return
+            # x and y are both vectors so this might qualifies for a GER
+            xv = x.dimshuffle(0)
+            yv = y.dimshuffle(1)
+            try:
+                bval = at.get_scalar_constant_value(b)
+            except NotScalarConstantError:
+                # b isn't a constant, GEMM is doing useful pre-scaling
+                return
 
-                if bval == 1:  # best case a natural GER
-                    rval = ger(z, a, xv, yv)
-                    return [rval]
-                elif bval == 0:  # GER on zeros_like should be faster than GEMM
-                    zeros = aet.zeros([x.shape[0], y.shape[1]], x.dtype)
-                    rval = ger(zeros, a, xv, yv)
-                    return [rval]
-                else:
-                    # if bval is another constant, then z is being usefully
-                    # pre-scaled and GER isn't really the right tool for the job.
-                    return
+            if bval == 1:  # best case a natural GER
+                rval = ger(z, a, xv, yv)
+                new_out = [rval]
+            elif bval == 0:  # GER on zeros_like should be faster than GEMM
+                zeros = at.zeros([x.shape[0], y.shape[1]], x.dtype)
+                rval = ger(zeros, a, xv, yv)
+                new_out = [rval]
+            else:
+                # if bval is another constant, then z is being usefully
+                # pre-scaled and GER isn't really the right tool for the job.
+                return
+            copy_stack_trace(node.outputs, new_out)
+            return new_out
 
 
 # TODO: delete this optimization when we have the proper dot->gemm->ger pipeline
@@ -1756,38 +1791,41 @@ def local_gemm_to_ger(fgraph, node):
 def local_dot22_to_ger_or_gemv(fgraph, node):
     """dot22 computing an outer-product -> GER."""
     if node.op == _dot22:
-        with inherit_stack_trace(node.outputs):
-            x, y = node.inputs
-            xb = x.broadcastable
-            yb = y.broadcastable
-            one = aet.as_tensor_variable(np.asarray(1, dtype=x.dtype))
-            zero = aet.as_tensor_variable(np.asarray(0, dtype=x.dtype))
-            if xb[1] and yb[0]:
-                # x and y are both vectors so this might qualifies for a GER
-                xv = x.dimshuffle(0)
-                yv = y.dimshuffle(1)
-                zeros = aet.zeros([x.shape[0], y.shape[1]], dtype=x.dtype)
-                rval = ger(zeros, one, xv, yv)
-                return [rval]
-            if xb[0] and yb[1]:
-                # x and y are both vectors so this qualifies for a sdot / ddot
-                # TODO: Aesara doesn't have a sdot, but gemv is better than _dot22
-                xv = x.dimshuffle(1)
-                zeros = aet.AllocEmpty(x.dtype)(1)
-                rval = gemv_no_inplace(zeros, one, y.T, xv, zero)
-                return [rval.dimshuffle("x", 0)]
-            if xb[0] and not yb[0] and not yb[1]:
-                # x is vector, y is matrix so try gemv
-                xv = x.dimshuffle(1)
-                zeros = aet.AllocEmpty(x.dtype)(y.shape[1])
-                rval = gemv_no_inplace(zeros, one, y.T, xv, zero)
-                return [rval.dimshuffle("x", 0)]
-            if not xb[0] and not xb[1] and yb[1]:
-                # x is matrix, y is vector, try gemv
-                yv = y.dimshuffle(0)
-                zeros = aet.AllocEmpty(x.dtype)(x.shape[0])
-                rval = gemv_no_inplace(zeros, one, x, yv, zero)
-                return [rval.dimshuffle(0, "x")]
+        x, y = node.inputs
+        xb = x.broadcastable
+        yb = y.broadcastable
+        one = at.as_tensor_variable(np.asarray(1, dtype=x.dtype))
+        zero = at.as_tensor_variable(np.asarray(0, dtype=x.dtype))
+        if xb[1] and yb[0]:
+            # x and y are both vectors so this might qualifies for a GER
+            xv = x.dimshuffle(0)
+            yv = y.dimshuffle(1)
+            zeros = at.zeros([x.shape[0], y.shape[1]], dtype=x.dtype)
+            rval = ger(zeros, one, xv, yv)
+            new_out = [rval]
+        elif xb[0] and yb[1]:
+            # x and y are both vectors so this qualifies for a sdot / ddot
+            # TODO: Aesara doesn't have a sdot, but gemv is better than _dot22
+            xv = x.dimshuffle(1)
+            zeros = at.AllocEmpty(x.dtype)(1)
+            rval = gemv_no_inplace(zeros, one, y.T, xv, zero)
+            new_out = [rval.dimshuffle("x", 0)]
+        elif xb[0] and not yb[0] and not yb[1]:
+            # x is vector, y is matrix so try gemv
+            xv = x.dimshuffle(1)
+            zeros = at.AllocEmpty(x.dtype)(y.shape[1])
+            rval = gemv_no_inplace(zeros, one, y.T, xv, zero)
+            new_out = [rval.dimshuffle("x", 0)]
+        elif not xb[0] and not xb[1] and yb[1]:
+            # x is matrix, y is vector, try gemv
+            yv = y.dimshuffle(0)
+            zeros = at.AllocEmpty(x.dtype)(x.shape[0])
+            rval = gemv_no_inplace(zeros, one, x, yv, zero)
+            new_out = [rval.dimshuffle(0, "x")]
+        else:
+            return
+        copy_stack_trace(node.outputs, new_out)
+        return new_out
 
 
 #################################
@@ -1799,15 +1837,19 @@ def local_dot22_to_ger_or_gemv(fgraph, node):
 blas_optdb = SequenceDB()
 
 # run after numerical stability optimizations (1.5)
-optdb.register("BlasOpt", blas_optdb, 1.7, "fast_run", "fast_compile")
+optdb.register("BlasOpt", blas_optdb, "fast_run", "fast_compile", position=1.7)
 # run before specialize (2.0) because specialize is basically a
 # free-for-all that makes the graph crazy.
 
 # fast_compile is needed to have GpuDot22 created.
 blas_optdb.register(
-    "local_dot_to_dot22", in2out(local_dot_to_dot22), 0, "fast_run", "fast_compile"
+    "local_dot_to_dot22",
+    in2out(local_dot_to_dot22),
+    "fast_run",
+    "fast_compile",
+    position=0,
 )
-blas_optdb.register("gemm_optimizer", GemmOptimizer(), 10, "fast_run")
+blas_optdb.register("gemm_optimizer", GemmOptimizer(), "fast_run", position=10)
 blas_optdb.register(
     "local_gemm_to_gemv",
     EquilibriumOptimizer(
@@ -1820,8 +1862,8 @@ blas_optdb.register(
         max_use_ratio=5,
         ignore_newtrees=False,
     ),
-    15,
     "fast_run",
+    position=15,
 )
 
 
@@ -1831,7 +1873,12 @@ blas_opt_inplace = in2out(
     local_inplace_gemm, local_inplace_gemv, local_inplace_ger, name="blas_opt_inplace"
 )
 optdb.register(
-    "InplaceBlasOpt", blas_opt_inplace, 70.0, "fast_run", "inplace", "blas_opt_inplace"
+    "InplaceBlasOpt",
+    blas_opt_inplace,
+    "fast_run",
+    "inplace",
+    "blas_opt_inplace",
+    position=70.0,
 )
 
 
@@ -1848,6 +1895,10 @@ class Dot22Scalar(GemmRelated):
     check_input = False
 
     def make_node(self, x, y, a):
+
+        if any(not isinstance(i.type, DenseTensorType) for i in (x, y, a)):
+            raise NotImplementedError("Only dense tensor types are supported")
+
         if a.ndim != 0:
             raise TypeError(Gemm.E_scalar, a)
         if x.ndim != 2:
@@ -1973,7 +2024,7 @@ def local_dot22_to_dot22scalar(fgraph, node):
         i_mul = [
             x.owner
             and x.owner.op == mul
-            and any([_as_scalar(x_i, dtype=d.dtype) for x_i in x.owner.inputs])
+            and any(_as_scalar(x_i, dtype=d.dtype) for x_i in x.owner.inputs)
             for x in node.inputs
         ]
         if not any(i_mul):
@@ -2000,9 +2051,7 @@ def local_dot22_to_dot22scalar(fgraph, node):
                 " matrix type"
             )
             return False
-        a = aet.cast(
-            _as_scalar(m.owner.inputs[scalar_idx], dtype=d.dtype), d.type.dtype
-        )
+        a = at.cast(_as_scalar(m.owner.inputs[scalar_idx], dtype=d.dtype), d.type.dtype)
         assert not a.type.ndim
         dot = _dot22scalar(d.owner.inputs[0], d.owner.inputs[1], a)
 
@@ -2040,7 +2089,7 @@ def local_dot22_to_dot22scalar(fgraph, node):
     o.remove(d)
     o.remove(s)
 
-    a = aet.cast(i_scalar[scalar_idx], d.type.dtype)
+    a = at.cast(i_scalar[scalar_idx], d.type.dtype)
     assert not a.type.ndim
     if len(o) == 0:
         return [_dot22scalar(d.owner.inputs[0], d.owner.inputs[1], a)]
@@ -2051,7 +2100,10 @@ def local_dot22_to_dot22scalar(fgraph, node):
 # must happen after gemm as the gemm optimizer don't understant
 # dot22scalar and gemm give more speed up then dot22scalar
 blas_optdb.register(
-    "local_dot22_to_dot22scalar", in2out(local_dot22_to_dot22scalar), 11, "fast_run"
+    "local_dot22_to_dot22scalar",
+    in2out(local_dot22_to_dot22scalar),
+    "fast_run",
+    position=11,
 )
 
 
@@ -2065,7 +2117,10 @@ class BatchedDot(COp):
     __props__ = ()
 
     def make_node(self, *inputs):
-        inputs = list(map(aet.as_tensor_variable, inputs))
+        inputs = list(map(at.as_tensor_variable, inputs))
+
+        if any(not isinstance(i.type, DenseTensorType) for i in inputs):
+            raise NotImplementedError("Only dense tensor types are supported")
 
         if len(inputs) != 2:
             raise TypeError(f"Two arguments required, but {len(inputs)} given.")
@@ -2084,7 +2139,7 @@ class BatchedDot(COp):
 
         dtype = aesara.scalar.upcast(*[input.type.dtype for input in inputs])
         # upcast inputs to common dtype if needed
-        upcasted_inputs = [aet.cast(input, dtype) for input in inputs]
+        upcasted_inputs = [at.cast(input, dtype) for input in inputs]
         broadcastable = (
             (inputs[0].type.broadcastable[0] or inputs[1].type.broadcastable[0],)
             + inputs[0].type.broadcastable[1:-1]
@@ -2444,9 +2499,9 @@ class BatchedDot(COp):
         # above code don't always return the right broadcast pattern.
         # This cause problem down the road. See gh-1461.
         if xgrad.broadcastable != x.broadcastable:
-            xgrad = aet.patternbroadcast(xgrad, x.broadcastable)
+            xgrad = at.patternbroadcast(xgrad, x.broadcastable)
         if ygrad.broadcastable != y.broadcastable:
-            ygrad = aet.patternbroadcast(ygrad, y.broadcastable)
+            ygrad = at.patternbroadcast(ygrad, y.broadcastable)
 
         return xgrad, ygrad
 
@@ -2567,7 +2622,7 @@ def batched_dot(a, b):
             dot products in terms of batched matrix-matrix dot products, so
             it may be possible to further optimize for performance.
     """
-    a, b = aet.as_tensor_variable(a), aet.as_tensor_variable(b)
+    a, b = at.as_tensor_variable(a), at.as_tensor_variable(b)
 
     if a.ndim == 0:
         raise TypeError("a must have at least one (batch) axis")
