@@ -7,9 +7,10 @@ from aesara import config, printing
 from aesara import scalar as aes
 from aesara.gradient import DisconnectedType
 from aesara.graph.basic import Apply, Variable
-from aesara.graph.op import COp, Op
-from aesara.graph.params_type import ParamsType
-from aesara.graph.type import Generic
+from aesara.graph.op import Op
+from aesara.link.c.op import COp
+from aesara.link.c.params_type import ParamsType
+from aesara.link.c.type import Generic
 from aesara.misc.safe_asarray import _asarray
 from aesara.printing import pprint
 from aesara.scalar.basic import BinaryScalarOp
@@ -33,6 +34,7 @@ from aesara.tensor.elemwise import (
 )
 from aesara.tensor.shape import shape
 from aesara.tensor.type import (
+    DenseTensorType,
     complex_dtypes,
     continuous_dtypes,
     discrete_dtypes,
@@ -1548,11 +1550,6 @@ def mean(input, axis=None, dtype=None, op=False, keepdims=False, acc_dtype=None)
         necessarily be the dtype of the output (in particular
         if it is a discrete (int/uint) dtype, the output will
         be in a float type). If None, then we use the same rules as `sum()`.
-
-    Notes
-    -----
-    For gpu, if you specify dtype=float32, everything will be done on the gpu.
-
     """
     input = as_tensor_variable(input)
     if op:
@@ -1674,10 +1671,10 @@ def var(input, axis=None, ddof=0, keepdims=False, corrected=False):
     # return the mean sqr
     two = constant(2, dtype=centered_input.dtype)
     if ddof == 0:
-        v = mean((centered_input ** two), axis, keepdims=keepdims)
+        v = mean((centered_input**two), axis, keepdims=keepdims)
     else:
         shp = shape(input) - ddof
-        v = sum((centered_input ** two), axis=axis, keepdims=keepdims)
+        v = sum((centered_input**two), axis=axis, keepdims=keepdims)
         for i in axis:
             v = true_div(v, shp[i])
 
@@ -2075,6 +2072,11 @@ def dense_dot(a, b):
     """
     a, b = as_tensor_variable(a), as_tensor_variable(b)
 
+    if not isinstance(a.type, DenseTensorType) or not isinstance(
+        b.type, DenseTensorType
+    ):
+        raise TypeError("The dense dot product is only supported for dense types")
+
     if a.ndim == 0 or b.ndim == 0:
         return a * b
     elif a.ndim > 2 or b.ndim > 2:
@@ -2780,7 +2782,7 @@ def ptp(a, axis=None):
 
 
 def power(x, y):
-    return x ** y
+    return x**y
 
 
 def logaddexp(*xs):
