@@ -11,10 +11,11 @@ from aesara import scalar as aes
 from aesara.configdefaults import config
 from aesara.gradient import DisconnectedType
 from aesara.graph.basic import Apply, Constant, Variable
-from aesara.graph.op import COp, Op
-from aesara.graph.params_type import ParamsType
+from aesara.graph.op import Op
 from aesara.graph.type import Type
 from aesara.graph.utils import MethodNotDefined
+from aesara.link.c.op import COp
+from aesara.link.c.params_type import ParamsType
 from aesara.misc.safe_asarray import _asarray
 from aesara.printing import Printer, pprint, set_precedence
 from aesara.scalar.basic import ScalarConstant
@@ -135,8 +136,8 @@ def as_index_constant(
 
 
 def as_index_literal(
-    idx: Union[Variable, slice, type(np.newaxis)]
-) -> Union[int, slice, type(np.newaxis)]:
+    idx: Optional[Union[Variable, slice]]
+) -> Optional[Union[int, slice]]:
     """Convert a symbolic index element to its Python equivalent.
 
     This is like the inverse of `as_index_constant`
@@ -623,7 +624,7 @@ def get_constant_idx(
     Example usage where `v` and `a` are appropriately typed Aesara variables :
     >>> b = a[v, 1:3]
     >>> b.owner.op.idx_list
-    (Scalar(int64), slice(Scalar(int64), Scalar(int64), None))
+    (ScalarType(int64), slice(ScalarType(int64), ScalarType(int64), None))
     >>> get_constant_idx(b.owner.op.idx_list, b.owner.inputs, allow_partial=True)
     [v, slice(1, 3, None)]
     >>> get_constant_idx(b.owner.op.idx_list, b.owner.inputs)
@@ -655,7 +656,7 @@ def get_constant_idx(
 
 
 def as_nontensor_scalar(a: Variable) -> aes.ScalarVariable:
-    """Convert a value to a `Scalar` variable."""
+    """Convert a value to a `ScalarType` variable."""
     # Since aes.as_scalar does not know about tensor types (it would
     # create a circular import) , this method converts either a
     # TensorVariable or a ScalarVariable to a scalar.
@@ -864,7 +865,7 @@ class Subtensor(COp):
     ):
         """
         The parameters c_prefix are there to allow reusing this
-        function on PyArray and GpuArray object.
+        function on PyArray object.
 
         This fct take as input the x.
 
@@ -1195,7 +1196,7 @@ class SubtensorPrinter(Printer):
         sidxs = []
         getattr(pstate, "precedence", None)
         for entry in idxs:
-            if isinstance(entry, aes.Scalar):
+            if isinstance(entry, aes.ScalarType):
                 with set_precedence(pstate):
                     sidxs.append(pstate.pprinter.process(inputs.pop()))
             elif isinstance(entry, slice):
@@ -1580,9 +1581,7 @@ class IncSubtensor(COp):
         # This method delegates much of the work to helper
         # methods. This method implements the main logic
         # but subclasses may override the helper methods
-        # to change the particulars, e.g. GpuIncSubtensor
-        # turns the view/copy operations on numpy arrays
-        # into the same operations on gpu arrays.
+        # to change the particulars.
 
         self.do_type_checking(node)
 
@@ -1598,8 +1597,8 @@ class IncSubtensor(COp):
         else:
             op_is_set = 0
         fail = sub["fail"]
-        view_ndim = node.inputs[0].ndim - np.sum(
-            [not isinstance(idx, slice) for idx in self.idx_list]
+        view_ndim = node.inputs[0].ndim - sum(
+            not isinstance(idx, slice) for idx in self.idx_list
         )
 
         copy_of_x = self.copy_of_x(x)
